@@ -2,8 +2,8 @@
 
 """ OneLogin_Saml2_IdPMetadataParser class
 
-Copyright (c) 2014, OneLogin, Inc.
-All rights reserved.
+Copyright (c) 2010-2018 OneLogin, Inc.
+MIT License
 
 Metadata class of OneLogin's Python Toolkit.
 
@@ -50,11 +50,11 @@ class OneLogin_Saml2_IdPMetadataParser(object):
 
         if xml:
             try:
-                dom = fromstring(xml)
+                dom = fromstring(xml, forbid_dtd=True)
                 idp_descriptor_nodes = OneLogin_Saml2_Utils.query(dom, '//md:IDPSSODescriptor')
                 if idp_descriptor_nodes:
                     valid = True
-            except:
+            except Exception:
                 pass
 
         if not valid:
@@ -124,7 +124,7 @@ class OneLogin_Saml2_IdPMetadataParser(object):
         """
         data = {}
 
-        dom = fromstring(idp_metadata)
+        dom = fromstring(idp_metadata, forbid_dtd=True)
 
         entity_desc_path = '//md:EntityDescriptor'
         if entity_id:
@@ -221,10 +221,10 @@ class OneLogin_Saml2_IdPMetadataParser(object):
         Will update the settings with the provided new settings data extracted from the IdP metadata
 
         :param settings: Current settings dict data
-        :type settings: string
+        :type settings: dict
 
         :param new_metadata_settings: Settings to be merged (extracted from IdP metadata after parsing)
-        :type new_metadata_settings: string
+        :type new_metadata_settings: dict
 
         :returns: merged settings
         :rtype: dict
